@@ -72,7 +72,7 @@ PlaylistsInFoldersProxy::data( const QModelIndex &idx, int role ) const
     if( idx.column() == 0 && isGroup( idx ) && role ==
         PlaylistBrowserNS::PlaylistBrowserModel::ActionRole )
     {
-        //wheter we use the list from m_deleteFolderAction or m_renameFolderAction does not matter
+        //whether we use the list from m_deleteFolderAction or m_renameFolderAction does not matter
         //they are the same anyway
         QPersistentModelIndexList actionList =
                 m_deleteFolderAction->data().value<QPersistentModelIndexList>();
@@ -108,8 +108,8 @@ PlaylistsInFoldersProxy::removeRows( int row, int count, const QModelIndex &pare
         }
 
         //is a playlist not in a folder
-        QModelIndex childIdx = mapToSource( index( row, 0, m_rootNode ) );
-        result = sourceModel()->removeRows( childIdx.row(), count, m_rootNode );
+        QModelIndex childIdx = mapToSource( index( row, 0, m_rootIndex ) );
+        result = sourceModel()->removeRows( childIdx.row(), count, m_rootIndex );
         if( result )
         {
             beginRemoveRows( parent, row, row + count - 1 );
