@@ -36,33 +36,27 @@ class MEDIADEVICECOLLECTION_EXPORT MediaDeviceCollectionLocation : public Collec
 {
     Q_OBJECT
     public:
-        MediaDeviceCollectionLocation( MediaDeviceCollection const *collection );
+        MediaDeviceCollectionLocation( MediaDeviceCollection *collection );
         virtual ~MediaDeviceCollectionLocation();
 
         virtual QString prettyLocation() const;
         virtual bool isWritable() const;
-        virtual bool remove( const Meta::TrackPtr &track );
 
     protected:
-        virtual void showDestinationDialog( const Meta::TrackList &tracks, bool removeSources,
-                                            const Transcoding::Configuration &configuration = Transcoding::Configuration() );
         virtual void getKIOCopyableUrls( const Meta::TrackList &tracks );
 
         /// Copies these tracks to the Collection using the Handler
         virtual void copyUrlsToCollection( const QMap<Meta::TrackPtr, KUrl> &sources,
-                                           const Transcoding::Configuration &configuration = Transcoding::Configuration() );
+                                           const Transcoding::Configuration &configuration );
 
         virtual void removeUrlsFromCollection( const Meta::TrackList &sources );
 
 
     private slots:
-        void slotDialogAccepted();
-        void slotDialogRejected();
         void copyOperationFinished( bool success );
         void removeOperationFinished();
 
     private:
-        QMap<Meta::TrackPtr, QString> m_destinations;
         MediaDeviceCollection *m_collection;
         Meta::MediaDeviceHandler *m_handler;
 };
