@@ -10,7 +10,7 @@
  *                                                                                      *
  * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
- * PARTICULAR PURPOSE. See the GNU General Public License for more details.              *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
  *                                                                                      *
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
@@ -22,12 +22,17 @@
 #include <QObject>
 #include <KUrl>
 
+class QScriptEngine;
+
+namespace AmarokScript {
+
 class InfoScript : public QObject
 {
     Q_OBJECT
 
     public:
-        InfoScript( const KUrl& scriptUrl );
+        InfoScript( const KUrl& scriptUrl, QScriptEngine *engine );
+
     public slots:
         QString scriptPath() const;
         QString scriptConfigPath( const QString& name ) const;
@@ -42,8 +47,9 @@ class IconEnum : public QObject
 {
     Q_OBJECT
     Q_ENUMS( StdSizes )
+
     public:
-        IconEnum() : QObject() { }
+        IconEnum( QObject *parent ) : QObject(parent) {}
       enum StdSizes {
           Small=16,
           SmallMedium=22,
@@ -54,4 +60,6 @@ class IconEnum : public QObject
       };
 };
 
+} // namespace AmarokScript
+
 #endif
