@@ -22,34 +22,10 @@ AmarokDockWidget::AmarokDockWidget( const QString & title, QWidget * parent, Qt:
     : QDockWidget( title, parent, flags )
     , m_polished( false )
 {
-    m_dummyTitleBarWidget = new QWidget();
+    m_dummyTitleBarWidget = new QWidget( this );
     connect( this, SIGNAL( visibilityChanged( bool ) ), SLOT( slotVisibilityChanged( bool ) ) );
 }
 
-void AmarokDockWidget::closeEvent( QCloseEvent* event )
-{
-    QDockWidget::closeEvent( event );
-    emit( layoutChanged() );
-}
-
-void AmarokDockWidget::resizeEvent( QResizeEvent* event )
-{
-    QWidget::resizeEvent( event );
-    emit( layoutChanged() );
-}
-
-void AmarokDockWidget::showEvent( QShowEvent* event )
-{
-    QWidget::showEvent( event );
-    emit( layoutChanged() );
-}
-
-void AmarokDockWidget::hideEvent( QHideEvent* event )
-{
-    QWidget::hideEvent( event );
-    emit( layoutChanged() );
-}
-
 void AmarokDockWidget::slotVisibilityChanged( bool visible )
 {
     // DEBUG_BLOCK
@@ -68,7 +44,6 @@ void AmarokDockWidget::ensurePolish()
 
 void AmarokDockWidget::setMovable( bool movable )
 {
-
     if( movable )
     {
         const QFlags<QDockWidget::DockWidgetFeature> features = QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetClosable;
@@ -81,9 +56,6 @@ void AmarokDockWidget::setMovable( bool movable )
         setTitleBarWidget( m_dummyTitleBarWidget );
         setFeatures( features );
     }
-
-
 }
 
-
 #include "AmarokDockWidget.moc"
